set echo on pagesize 999 tab off

-- Walk the data according to the index on the sequence based value
-- Show how the distribution of data across blocks
select * from (
  select /*+ index(t1 t1_idx1) */
        dbms_rowid.rowid_block_number(rowid) as block_no
      , t1.batch_id
      , t1.a_value
      , t1.run_id
  from
        t1
  where
        batch_id is not null
  and   a_value is not null
  and   a_random is not null
)
where rownum <= 100
;

-- Check the actual amount of work when using the index->table lookup for a index range scan
set echo on

alter system flush buffer_cache;

alter session set statistics_level = all;

-- Perform scan of 10% of data (when sticking to defaults, 100 batches, four concurrent processes)
-- The table data is supposed to be very well clustered in relation to this index
select * from (
select /*+ index(t1 t1_idx1) */ * from t1
--where batch_id between 1 and 10
where a_value between 1 and 4000
) where rownum > 1;

@xplan_extended_display_cursor

set echo on

alter system flush buffer_cache;

alter session set statistics_level = all;

-- Perform scan of 10% of data (when sticking to defaults, 100 batches, four concurrent processes)
-- The table data is supposed to be very scattered in relation to this index
select * from (
select /*+ index(t1 t1_idx2) */ * from t1 where a_random between 1 and 100
) where rownum > 1;

@xplan_extended_display_cursor
