-- See the effect of ASSM / MSSM on clustering factor
-- You need two tablespace with ASSM and MSSM to see the difference
-- Run this experiment twice, once with ASSM, once with MSSM tablespace
-- Run "concurrent_execution_inserts_check_results.sql" to see more details about the difference
--
-- !! Note !!
-- ! The script assumes that the USER has lower(username) as password !
--
set echo on

prompt Tablespace
accept tblspace
rem define tblspace = test_8k

-- If you want a hash partitioned table, just press ENTER
-- If you don't want a partitioned table, enter "--" to comment this out
prompt Table Partition
accept if_t_partition
rem define if_t_partition = "--"

-- If you want a locally partitioned index, just press ENTER
-- If you don't want a locally partitioned index, enter "--" to comment this out
prompt Index Partition
accept if_i_partition
rem define if_i_partition = "--"

drop table t1 purge;

drop sequence seq_t1_run_id;

drop sequence seq_t1_seq_id;

create table t1 (
run_id    integer                        not null, /* identify the process inserting the data */
batch_id  integer                        not null, /* represents clustered data, could also be a (arriving) date */
a_value   number                             null, /* represents sequence based data */
a_random  number                             null, /* represents randomly scattered data */
a_date    timestamp default systimestamp not null, /* represents the insert timestamp */
filler    char(1)   default 'x'          not null  /* can be used to size the row as required */
)
&if_t_partition partition by hash (a_value) partitions 4
tablespace &tblspace
pctfree 90 pctused 10;

create sequence seq_t1_run_id;

create sequence seq_t1_seq_id cache 10000;

truncate table t1;

-- Adjust as required, the 4 is the concurrency level, shouldn't be more than CPUs available
-- 100 is the number of batch inserts - each batch is 100 rows. This makes 10,000 rows with this configuration
-- So each process inserts 10,000 rows each concurrently
@concurrent_execution_launch_slaves 4 insert_no_wait 100

prompt Press return to continue when sessions have completed

accept x

/* a sample index */
create index t1_idx1 on
t1 (
--batch_id,
a_value
)
tablespace &tblspace
&if_i_partition local
;

/* a sample index */
create index t1_idx2 on
t1 (
a_random
)
tablespace &tblspace
&if_i_partition local
;

exec dbms_stats.gather_table_stats(null, 't1', estimate_percent=>null, cascade=>true)

select
        i.index_name
      , i.num_rows
      , i.clustering_factor
      , t.blocks
from
        user_indexes i
      , user_tables t
where
        t.table_name = 'T1'
and     i.table_name = t.table_name;
