define lock_name = 'CONCURRENT_EXECUTION'

-- The following is a hack to use default
-- values for defines
column 1 new_value 1
column 2 new_value 2
column 3 new_value 3

select
        '' as "1"
      , '' as "2"
      , '' as "3"
from
        dual
where
        rownum = 0;

column iter new_value iter noprint
column which_slave new_value which_slave noprint
column pwd new_value pwd noprint
column optional_param new_value optional_param noprint

-- By default start 4 concurrent sessions
-- Use the "insert_no_wait" script by default
select
        nvl('&1', '4')               as iter
      , nvl('&2', 'insert_no_wait')  as which_slave
      , '&3'                         as optional_param
      , lower('&_user')              as pwd
from
        dual;

-- Save current settings
store set .settings replace

spool off

-- Create a script to launch as many slaves as requested
set echo off heading off feedback off long 10000000 longchunksize 1000000 trimspool on newpage none pagesize 0 termout off verify off tab off timing off linesize 200

spool launch_concurrent_slaves.sql

select
        'host start sqlplus &_user/&pwd@&_connect_identifier @&which_slave ' || rownum || ' &optional_param' as task
from
        dual
connect by
        level <= &iter;

spool off

-- Append is only supported in recent SQL*Plus releases
spool concurrent_execution.log append

-- Restore settings
@.settings

-- Request an exclusive user lock
-- This allows a synchronised start of all slaves being launched
declare
  n         number;
  m_handle  varchar2(60);
begin
  dbms_lock.allocate_unique(
    lockname    => '&lock_name',
    lockhandle  => m_handle
  );
  n := dbms_lock.request(
    lockhandle        => m_handle,
    lockmode          => dbms_lock.x_mode,
    timeout           => dbms_lock.maxwait,
    release_on_commit => true
  );
end;
/

-- Launch the slaves
@launch_concurrent_slaves

PAUSE  Hit CTRL+C to cancel, ENTER to continue...
--@wait 2

-- Start the execution of the slaves
commit;

-- Request an exclusive user lock
-- This allows this session to continue once all slaves have completed their operation

set timing on

declare
  n         number;
  m_handle  varchar2(60);
begin
  dbms_lock.allocate_unique(
    lockname    => '&lock_name',
    lockhandle  => m_handle
  );
  n := dbms_lock.request(
    lockhandle        => m_handle,
    lockmode          => dbms_lock.x_mode,
    timeout           => dbms_lock.maxwait,
    release_on_commit => true
  );
end;
/

commit;

spool off
