set echo on timing on time on

drop table t;

purge table t;

-- This is a data distribution that doesn't work well with Oracle histograms
-- depending on the questions you ask about the data
--
-- 100 very popular values (70,000 occurrences) along with 3,000,000 unpopular values
-- and a huge gap inbetween on top
create table t
pctfree 0
-- compress
as
select
          rownum as id
        , case
          when rownum <= 7000000 then ceil(rownum / 70000) * 1000
          else rownum
          end as skewed_number
from    (
          select  /*+ cardinality(10000) */
                  null
          from
                  dual
          connect by
                  level <= 10000
        ) v1
      , (
          select  /*+ cardinality(1000) */
                  null
          from
                  dual
          connect by level <= 1000
        ) v2
;

-- create index t_idx on t (skewed_number) reverse pctfree 0;

-- select * from user_tab_col_statistics where table_name = 'T';

-- select * from user_indexes where table_name = 'T';

-- select * from sys.col_usage$ where obj# = (select object_id from user_objects where object_name = 'T')

-- select * from user_tab_histograms where table_name = 'T';

set echo on timing on

-- No histograms => bad estimates
-- Run repeatedly below EXPLAIN PLANs after refreshing statistics
exec dbms_stats.gather_table_stats(null, 't', method_opt => 'for all columns size 1')

-- The actual data distribution
select * from (
select skewed_number, count(*) cnt from t
group by skewed_number
order by skewed_number
)
where rownum <= 110
;

-- Use default options, shows that SIZE AUTO will generate the same histograms if query workload has been generated (EXPLAIN PLAN below is sufficient)
exec dbms_stats.gather_table_stats(null, 't')

-- Explicitly ask for histograms
-- Run repeatedly below EXPLAIN PLANs after refreshing statistics, and you'll see huge differences in the estimates
exec dbms_stats.gather_table_stats(null, 't', method_opt => 'for all columns size 1 for columns size 250 skewed_number')

-- Compute statistics
exec dbms_stats.gather_table_stats(null, 't', method_opt => 'for all columns size 1 for columns size 250 skewed_number', estimate_percent=>null)

-- Compute statistics faster if you have enough memory and CPUs
set echo on timing on

alter session set workarea_size_policy = manual;

alter session set sort_area_size = 1400000000;

alter session set sort_area_size = 1400000000;

alter session set hash_area_size = 1400000000;

alter session set hash_area_size = 1400000000;

alter session set "_small_table_threshold" = 100000000;

select /*+ parallel(t, 4) */ count(*) from t;

exec dbms_stats.gather_table_stats(null, 't', method_opt => 'for all columns size 1 for columns size 250 skewed_number', degree=>4, estimate_percent=>null)

-- Height Balanced Histograms are well suited for some broader range queries
-- If you ask for ranges that are within a bucket you still might end up with bad estimates
-- => A Height Balanced Histogram doesn't know anything about the contents of a bucket if the value isn't popular (covers more than 1 bucket)
explain plan set statement_id = 'Non-existing range'
for
select count(*) from t where skewed_number between 6000000 and 6100000;

select * from table(dbms_xplan.display);

explain plan set statement_id = 'Range of popular values'
for
select count(*) from t where skewed_number between 1000 and 100000;

select * from table(dbms_xplan.display);

-- alter session set optimizer_features_enable = '10.2.0.4';

-- alter session set "_fix_control" = '5483301:off';

explain plan set statement_id = '1000'
for
select count(*) from t where skewed_number = 1000;

explain plan set statement_id = '2000'
for
select count(*) from t where skewed_number = 2000;

explain plan set statement_id = '3000'
for
select count(*) from t where skewed_number = 3000;

explain plan set statement_id = '4000'
for
select count(*) from t where skewed_number = 4000;

explain plan set statement_id = '5000'
for
select count(*) from t where skewed_number = 5000;

explain plan set statement_id = '6000'
for
select count(*) from t where skewed_number = 6000;

explain plan set statement_id = '7000'
for
select count(*) from t where skewed_number = 7000;

explain plan set statement_id = '8000'
for
select count(*) from t where skewed_number = 8000;

explain plan set statement_id = '9000'
for
select count(*) from t where skewed_number = 9000;

explain plan set statement_id = '10000'
for
select count(*) from t where skewed_number = 10000;

explain plan set statement_id = '10000000'
for
select count(*) from t where skewed_number = 10000000;

select statement_id, cardinality from plan_table where id = 2;
